local listeners;
local GetWatchedFactionInfo_Orig;
local bar_name;
local bar_max;
local bar_value;

function XPMonitor_OnLoad()
	this:RegisterEvent("CHAT_MSG_ADDON");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	SLASH_XPMonitor1 = "/xpm";
	SLASH_XPMonitor2 = "/XPMonitor";
	SlashCmdList["XPMonitor"] = XPMonitor_HandleSlashCmd;

	GetWatchedFactionInfo_Orig = GetWatchedFactionInfo;
	GetWatchedFactionInfo = XPMonitor_GetWatchedFactionInfo;
	listeners = 0;
end

function XPMonitor_Using_RepBar()
	return (bar_name ~= nil);
end

function XPMonitor_GetWatchedFactionInfo()
	if bar_name == "***pet***" then
		return UnitName("pet"), min(8,floor((((bar_value * 100) / bar_max) / 12)) + 1), 0, bar_max, bar_value;
	elseif bar_name ~= nil then
		return bar_name, min(8,floor((((bar_value * 100) / bar_max) / 12)) + 1), 0, bar_max, bar_value;
	else
		return GetWatchedFactionInfo_Orig();
	end
end

function XPMonitor_Set_GMOTD()
	local total = 0;
	local num = GetNumGuildMembers(true);
	for index = 1, num do
		local _, _, _, level, _, _, _, _, online, _ = GetGuildRosterInfo(index);
		total = total + level;
	end
	GuildSetMOTD(string.format("Welcome! Average level is %.2f.", (total / num)));
end

function XPMonitor_HandleSlashCmd(arrrgs)
	if strlen(arrrgs) == 0 then
		XPMonitor_PrintHelp();
		return;
	end	
	args = explode(" ", arrrgs);
	if args[1] == "create" then
		XPMonitor_CreateXPMonitor(args[2]);
	elseif args[1] == "createbar" then
		XPMonitor_CreateBarMonitor(args[2]);
	elseif args[1] == "createall" then
		XPMonitor_CreateAllMonitors();
	elseif args[1] == "createbarpet" then
		XPMonitor_CreateBarMonitor("***pet***");
		XPMonitor_Handle_Pet_Update();
	elseif args[1] == "createpet" then
		XPMonitor_CreatePetMonitor();
	elseif args[1] == "delete" then
		XPMonitor_RemoveXPMonitor(args[2]);
	elseif args[1] == "deletepetbar" then
		XPMonitor_RemovePetMonitor();
	else
		DEFAULT_CHAT_FRAME:AddMessage("Unrecognized command: " .. args[1]);
	end
end

function XPMonitor_CreateBarMonitor(plyr)
	SetWatchedFactionIndex(99); -- force an invalid faction so we can borrow the bar
	local player = strupper(strsub(plyr, 1, 1)) .. strsub(plyr, 2);
	bar_name = player;
	if plyr ~= "***pet***" then
		SendAddonMessage("XPM", "SUBSCRIBE", "WHISPER", player);
	end
end

function d(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

function XPMonitor_CreateAllMonitors()
	GuildRoster();
	local me = UnitName("player");
	local num = GetNumGuildMembers(true);
	for index = 1, num do
		local name, _, _, level, _, _, _, _, online, _ = GetGuildRosterInfo(index);
		if online == 1 and level ~= 80 and name ~= me then
			XPMonitor_CreateXPMonitor(name);
		end
	end
end

function XPMonitor_CreateXPMonitor(plyr)
	local player = strupper(strsub(plyr, 1, 1)) .. strsub(plyr, 2);
	if getglobal("XPMonitor_Bar_" .. player) ~= nil then
		getglobal("XPMonitor_Bar_" .. player):Show();
		SendAddonMessage("XPM", "SUBSCRIBE", "WHISPER", player);
		return;
	end
	local bar = CreateFrame("Frame", "XPMonitor_Bar_" .. player, UIParent, "XPMonitor_Bar");
	bar:SetResizable(true);
	bar:SetFrameStrata("DIALOG");
	bar:SetMovable(true);
	bar:SetToplevel(true);
	bar:SetClampedToScreen(true);
	bar:SetPoint("CENTER", UIParent);
	local titleRegion = bar:CreateTitleRegion();
	titleRegion:SetAllPoints(bar);
	--title
	local title = bar:CreateFontString('ARTWORK');
	title:SetFontObject('GameFontHighlightSmall');
	title:SetText(player);
	title:SetPoint('TOP', bar, 'TOP', 0, 10);
	bar:Show();
	local display = getglobal("XPMonitor_Bar_" .. player .. "_Display");
	local titleRegionD = display:CreateTitleRegion()
	titleRegionD:SetAllPoints(display)
	local titleD = display:CreateFontString('ARTWORK')
	titleD:SetFontObject('GameFontHighlightSmall')
	titleD:SetText("0/0 0%")
	titleD:SetPoint('CENTER', display, 'CENTER', 0, 0)
	display:SetMovable(false);
	display:SetResizable(false);
	display:RegisterForDrag();
	SendAddonMessage("XPM", "SUBSCRIBE", "WHISPER", player);
end

function XPMonitor_RemoveXPMonitor(plyr)
	if plyr == "xpbar" then
		SetWatchedFactionIndex(-1); -- force a different invalid faction to clear
		bar_name = nil;
		return;
	end
	local player = strupper(strsub(plyr, 1, 1)) .. strsub(plyr, 2);
	if getglobal("XPMonitor_Bar_" .. player) == nil then
		d("Player " .. player .. " is not being monitored!");
		return;
	end
	SendAddonMessage("XPM", "UNSUBSCRIBE", "WHISPER", player);
	getglobal("XPMonitor_Bar_" .. player):Hide();
end

function XPMonitor_RemovePetMonitor()
	if getglobal("XPMonitor_Bar_Pet") == nil then
		d("You weren't watching your pet!");
		return;
	end
	getglobal("XPMonitor_Bar_Pet"):Hide();
end

function XPMonitor_OnEvent()
	if event == "CHAT_MSG_ADDON" and arg1 == "XPM" then
		if arg2 == "SUBSCRIBE" then
			XPMonitor_Add_Listener(arg4);
		elseif arg2 == "UNSUBSCRIBE" then
			XPMonitor_Remove_Listener(arg4);
		elseif strsub(arg2, 0, 6) == "UPDATE" then
			XPMonitor_Handle_Update(arg2, arg4);
		elseif arg2 == "AVAILABLE" then
			if getglobal("XPMonitor_Bar_" .. arg4) ~= nil then
				SendAddonMessage("XPM", "SUBSCRIBE", "WHISPER", arg4);
			end
		end
	elseif event == "PLAYER_XP_UPDATE" then
	  XPMonitor_Send_Update();
	elseif event == "PLAYER_ENTERING_WORLD" then
		SendAddonMessage("XPM", "AVAILABLE", "GUILD");
	elseif event == "UNIT_PET_EXPERIENCE" then
		XPMonitor_Handle_Pet_Update();
	end
end

function XPMonitor_CreatePetMonitor()
	local petname = UnitName("pet");
	if petname == nil then
		d("You don't have a pet out!");
		return;
	end
	if getglobal("XPMonitor_Bar_Pet") ~= nil then
		getglobal("XPMonitor_Bar_Pet"):Show();
		this:RegisterEvent("UNIT_PET_EXPERIENCE")
		return;
	end
	local bar = CreateFrame("Frame", "XPMonitor_Bar_Pet", UIParent, "XPMonitor_Bar");
	bar:SetResizable(true);
	bar:SetFrameStrata("DIALOG");
	bar:SetMovable(true);
	bar:SetToplevel(true);
	bar:SetClampedToScreen(true);
	bar:SetPoint("CENTER", UIParent);
	local titleRegion = bar:CreateTitleRegion();
	titleRegion:SetAllPoints(bar);
	--title
	local title = bar:CreateFontString('ARTWORK');
	title:SetFontObject('GameFontHighlightSmall');
	title:SetText(petname .. " (pet)");
	title:SetPoint('TOP', bar, 'TOP', 0, 10);
	bar:Show();
	local display = getglobal("XPMonitor_Bar_Pet_Display");
	local titleRegionD = display:CreateTitleRegion()
	titleRegionD:SetAllPoints(display)
	local titleD = display:CreateFontString('ARTWORK')
	titleD:SetFontObject('GameFontHighlightSmall')
	titleD:SetText("0/0 0%")
	titleD:SetPoint('CENTER', display, 'CENTER', 0, 0)
	display:SetMovable(false);
	display:SetResizable(false);
	display:RegisterForDrag();
	this:RegisterEvent("UNIT_PET_EXPERIENCE")
	XPMonitor_Handle_Pet_Update(); -- since we aren't firing a SUBSCRIBE to be responded to, force an update here
end

function XPMonitor_Handle_Pet_Update()
	local currXP, nextXP = GetPetExperience();
	if bar_name == "***pet***" then
		bar_max = nextXP;
		bar_value = currXP;
		ReputationWatchBar_Update();
	end
	
	local bar = getglobal("XPMonitor_Bar_Pet_Display");
	if bar == nil then return; end
	local _,txt = bar:GetRegions();
	bar:SetMinMaxValues(0, nextXP);
	bar:SetValue(currXP);
	local col = .25 + .75*currXP/nextXP
	bar:SetStatusBarColor(0, 1-col, col, col);
	txt:SetText(format("%d/%d : %.1f%%", currXP, nextXP, 100*currXP/nextXP));

end

function XPMonitor_Handle_Update(message, sender)
	if sender == bar_name then
		values = explode(" ", message);
		bar_max = values[3];
		bar_value = values[2];
		ReputationWatchBar_Update();
	end
	local bar = getglobal("XPMonitor_Bar_" .. sender .. "_Display");
	if bar == nil then return; end
	local _,txt = bar:GetRegions();
	values = explode(" ", message);
	bar:SetMinMaxValues(0, values[3]);
	bar:SetValue(values[2]);
	local col = .25 + .75*values[2]/values[3]
	bar:SetStatusBarColor(0, 1-col, col, col);
	txt:SetText(format("%d/%d : %.1f%%", values[2], values[3], 100*values[2]/values[3]));
end

function XPMonitor_Send_Update()
		SendAddonMessage("XPM", "UPDATE " .. UnitXP("player") .. " " .. UnitXPMax("player"), "GUILD");
		SendAddonMessage("XPM", "UPDATE " .. UnitXP("player") .. " " .. UnitXPMax("player"), "PARTY");
end

function XPMonitor_Add_Listener(player)
	if listeners == 0 then
		this:RegisterEvent("PLAYER_XP_UPDATE");
	end
	
	listeners = listeners + 1;
	SendAddonMessage("XPM", "UPDATE " .. UnitXP("player") .. " " .. UnitXPMax("player"), "WHISPER", player);
end

function XPMonitor_Remove_Listener(player)
	listeners = listeners - 1;
	if listeners < 0 then listeners = 0 end
	if listeners == 0 then
		this:UnregisterEvent("PLAYER_XP_UPDATE");
	end
end

-- explode(seperator, string)
function explode(sep,p)
	local t,ll,l
	t={}
	ll=0
	while true do
		l=string.find(p,sep,ll+1,true) -- find the next d in the string
		if l~=nil then -- if "not not" found then..
			table.insert(t, string.sub(p,ll,l-1)) -- Save it in our array.
			ll=l+1 -- save just after where we found it for searching next time.
		else
			table.insert(t, string.sub(p,ll)) -- Save what's left in our array.
			break -- Break at end, as it should be, according to the lua manual.
		end
	end
	return t
end
